/*
 * Neural network stuff.
 * Definitions and such.
 * This is a simple fully connected feed-forward network
 */

#ifndef __NNET_H__
#define __NNET_H__

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#define TRUE_LIMIT 0.95
#define FALSE_LIMIT 0.05

struct nn_unit {
    double value;
    double error;
    double thresh;
    double *weights;
};
struct nn_layer {
    int prev_units;
    int our_units;
    int next_units;
    struct nn_unit *units;
};

/* Notice: Input layer is layer 0 */
struct nn_net {
    int num_layers;
    int input_units;
    int output_units;
    struct nn_layer *layers;
};

/* 
 * Initialise a neural network.
 * 'layers' is the number of layers. minimum is 2, one input and one output
 * 'sizes' is an array of the size(in units, perceptrons, whatever) of each 
 * layer 
 */
struct nn_net *nn_init(int layers, int *sizes);

/*
 * This simply deallocates all memory associated with the net
 */
void nn_free(struct nn_net *nnet);

/*
 * This function takes the net, and an input-vector and runs
 * the vector through the net. It returns the result it gets
 */
double *nn_compute(struct nn_net *nnet, double *input);

/*
 * Teaches the net by running the input through it, and then
 * uses back-propagation to uptade the weights
 * Returns the sum of the squares of the errors, ie 
 * sum( (output[i] - result[i])^2 ) for all output nodes i.
 */
double nn_teach(struct nn_net *nnet, double *input, double *output, 
		double rate);

#endif /* __NNET_H__ */



